#################################################################################
# Created       : Martin Foster
# Modified      : 19-Sep-2003
#################################################################################
#
# Post - Module part of Ethereal Realms, designed to display posts based on 
#        information
# Copyright (C) 2000-2004  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 29-26503 TWP RD 511
#                 Spruce Grove, Alberta
#                 T7Y 1G4
#
#################################################################################

package eRealms::Post;						# Package name

use strict;							# Strict variable enforcement
use eRealms::Crypto;						# Cryptographic handle

#################################################################################
# Sub-Routines
#################################################################################

#####################
# Constructor
#
# Allows to class to be exported and thus spawn objects as a result

sub new
{
	#####################
	# Data Members
        my $class = shift;					# Class instance

	my $crypt;						# Cryptographic handle
        my $self;						# Self refereincing datatype


	#####################
	# Program Area

	# New instance
	$crypt = new eRealms::Crypto;

        # Define parameters
        $self = {
         CRYPT  => $crypt
        };

        # Bless $self into designated class
        bless $self, $class;

        # Return self
        return $self;
}


#####################
# Get Ignore
#
# This is in addition to the standard ignore handler.  Meaning that it
# will be used to block out a specific images.

sub GetIgnore
{
	#####################
	# Data members
	my $self      = shift;					# Self reference
	my $database  = shift;					# Database handle
	my $param     = shift;					# Parameter list
	my $ignore    = shift;					# Ignore handler

	my $puppet;						# Puppet name
	my $statement;						# Query handle
	my $type;						# Ignore type


	#####################
	# Program area

	# Reset
	%{$ignore} = {};


	# Prepare and execute
	$database->Pull(\$statement, "SELECT
                 PuppetName,
		 PuppetIgnore
                FROM PuppetIgnore
                WHERE PuppeteerLogin=?
                  AND PuppetIgnore IN ('visual','optics','behead')
                ORDER BY PuppetName",

	 $param->{'USER'}
	);

	# Loop and collect
	while (($puppet, $type) = $statement->fetchrow())
	{
		# Insert into a hash
		$ignore->{$puppet} = $type;
	}

	# Finish off
	$statement->finish();
}


#####################
# Get Posts
#
# Using basic information about the user will return a Statement handle (more 
# lightweight then an array) that will be used in the display the information
# gathered.

sub GetPosts
{
	#####################
	# Data members
	my $self      = shift;					# Self reference
	my $database  = shift;					# Database handle
	my $statement = shift;					# Statement handle (reference)
	my $param     = shift;					# Parameter list
	my $realm     = shift;					# Reference to realm hash

	my $chat;						# Chat mode
	my $format;						# Formatting used	
	my $privacy;						# Privacy feature
	my $results;						# Returned results
	my $start;						# Start point for posts

	my $range;						# Range
	my $scope;						# Scope of ignore
	my $select;						# Selection of posts

	my @args;						# Additional arguments

	#####################
	# Program area

	# Determine action 
	# Scrollback	
	if (defined($param->{'SCROLLBACK'}))
	{ 
		# Time of delay
		if (($param->{'SCROLLBACK'} !~ /^\d+$/)
		 && (length($param->{'SCROLLBACK'}) > 1))
		{
			# Cleanup param to be used
			$param->Cleanup('NUM');
		}

		# Position specific
		elsif ($param->{'SCROLLBACK'} =~ /^\d+$/)
		{
			# Safety
			$param->{'SCROLLBACK'} = ($param->{'SCROLLBACK'} > 500) ? 500 : $param->{'SCROLLBACK'};
			$param->{'SCROLLBACK'} = ($param->{'SCROLLBACK'} < 10)  ? 10  : $param->{'SCROLLBACK'};


			# Prepare and execute
			# Selects all post ID's for actual posts
			($param->{'NUM'}) = $database->DataGet("SELECT PostIDNumber
				FROM Post
				WHERE RealmName=?
				  AND PostClass IN ('general','play')
				  AND PostTo='all'
				ORDER BY PostIDNumber DESC OFFSET ? LIMIT 1",

			 $param->{'ROOM'},
			 $param->{'SCROLLBACK'}
			);


			# Fetch minimum number
			unless (defined($param->{'NUM'}))
			{
				# Fetch this specifically
				($param->{'NUM'}) = $database->DataGet("SELECT MIN(PostIDNumber)
					FROM Post
					WHERE RealmName=?
					  AND PostClass IN ('general','play')
					  AND PostTo='all'", 

				 $param->{'ROOM'},
				);
			}


			# Always define if not enough posts
			$param->{'NUM'} = (defined($param->{'NUM'})) ? $param->{'NUM'} : 1;
		}

		# Cleanup
		$param->Cleanup('SCROLLBACK');
	}	


	# Format handling
	# Retreive formatting used
	($format, $chat, $privacy) = $database->DataGetFormatting($param->{'USER'});


	# Formatting
	# Full formatting
	if ($format eq 'complete')	
	{ $format = 'PostFullFormat'; }

	# Spartan formatting
	elsif ($format eq 'spartan')
	{ $format = 'PostPartialFormat'; }

	# Full less images
	elsif ($format eq 'imageless')
	{ $format = 'PostImagelessFormat'; }


	# Add in arguments
	push(@args, $param->{'USER'}, $param->{'USER'}, $param->{'USER'});


	# Privacy filter
	# Full privacy
	if ($privacy eq 'yes')
	{
		# Form query
		$select = 'AND Post.RealmName=?';
		$range  = 'AND Post.PostClass <> ?';

		# Add in argument
		push(@args, 'private');
	}

	# Restrict to realms
	elsif ($privacy eq 'global')
	{
		# Form query
		$select = 'AND Post.RealmName=?';
		$range  = '';
	}

	# Default lack of privacy
	else
	{
		# Form query
		$select = 'AND (Post.RealmName=? OR Post.PostTo=?)';
		$range  = '';

		# Add in argument
		push(@args, $param->{'USER'});
	}


	# Time based searches
	unless (defined($param->{'NUM'}))
	{
		# Calculate delay
		# Have database fetch it
		($start) = $database->DataGetPostID();

		# Location based search
		$param->{'NUM'} = (defined($start)) ? $start : 1;
	}


	# Retreive privs
	($scope) = $database->DataGetScope($param->{'USER'});

	# Only if supervisory or moderator
	if ($scope eq 'no')
	{
		# Retreive access
		my ($choice) = $database->DataGetPuppeteerControl($param->{'ROOM'}, $param->{'USER'});

		# Disabling of local and global ignores
		if (defined($choice))
		{
			# Remove two entries
			shift(@args);
			shift(@args);

			# Specific data select
			$database->Pull($statement, "SELECT
				 Post.PostIDNumber, 
				 Post.$format, 
				 Post.PuppeteerLogin, 
				 Post.PuppetName,
				 Post.PostCmd, 
				 Post.PostClass 
				FROM Post 
				WHERE Post.PostIDNumber > ?::INT $select
				  AND (Post.PostTo='all' OR Post.PostTo=?) 
	      		    $range ORDER BY Post.PostIDNumber LIMIT 100",

			 $param->{'NUM'},
			 $param->{'ROOM'},
			 @args
			);

			# Return
			return 1;
		}	
	}


	# Position based search
	# Initialize and execute
	$database->Pull($statement, "SELECT
		 Post.PostIDNumber, 
		 Post.$format, 
		 Post.PuppeteerLogin, 
		 Post.PuppetName,
		 Post.PostCmd, 
		 Post.PostClass 
		FROM Post 
		WHERE Post.PostIDNumber > ?::INT $select
		  AND (Post.PostTo='all' OR Post.PostTo=?) 
                         AND (NOT EXISTS (SELECT PuppetIgnore.PuppetLogin
			FROM PuppetIgnore
			WHERE PuppetIgnore.PuppetIgnore='global'
			  AND PuppetIgnore.PuppeteerLogin=?
			  AND PuppetIgnore.PuppetLogin=Post.PuppeteerLogin)
		   OR Post.PuppeteerLogin IS NULL)
		  AND (NOT EXISTS (SELECT PuppetIgnore.PuppetName
			FROM PuppetIgnore
			WHERE PuppetIgnore.PuppetIgnore='single'
			  AND PuppetIgnore.PuppeteerLogin=?
			  AND PuppetIgnore.PuppetName=Post.PuppetName)
		   OR Post.PuppetName IS NULL)
	          $range ORDER BY Post.PostIDNumber LIMIT 100",

		 $param->{'NUM'},
		 $param->{'ROOM'},
		 @args
		);

	# Return
	return 1;
}


#####################
# Handle Bio
#
# Is used to take care of the biographical links listed on the posts
# through the use of gummies.

sub HandleBio
{
	#####################
	# Data members
	my $self     = shift;					# Self reference
	my $cgi      = shift;					# CGI handle
	my $inline   = shift;					# Pre-generated link
	my $post     = shift;					# Post to modify

	my $crypt    = $self->{'CRYPT'};			# Cryptographic handling
	my $sec;						# Secured line

	my %bio;						# Parameter hash 


	#####################
	# Program area

	# Determine existence
	if (${$post} =~ /\?SEC\=(\w+)\"/)
	{
		# Secure line
		$sec = $1;

		# Decrypt parameter
		$crypt->DecryptParam($sec, \%bio);
		

		# Confirm character and pool exist
		if ((defined($bio{'BCHAR'}))
		 && (defined($bio{'BPOOL'})))
		{
			# Escape characters
			$bio{'BCHAR'} = $cgi->escape($bio{'BCHAR'});
			$bio{'BPOOL'} = $cgi->escape($bio{'BPOOL'});

			# Search and replace
			${$post} =~ s/\?SEC\=$sec/$inline\&BCHAR\=$bio{'BCHAR'}\&BPOOL\=$bio{'BPOOL'}/s;
		}
	}
}


#####################
# Handle Ignore
#
# Basically allows someone to pull off an actual image if it exists
# and replace it with a link instead.

sub HandleIgnore
{
	#####################
	# Data members
	my $self     = shift;					# Self reference
	my $realm    = shift;					# Realm handle
	my $post     = shift;					# Post to modify
	my $type     = shift;					# Type of ignore
	my $puppet   = shift;					# Puppet name


	#####################
	# Program area

	# Is there a need
	if (defined($type))
	{
		# Type of filter
		# Visual
		if ($type eq 'visual')
		{
			# Pull image address and post as link
			${$post} =~ s/<IMG SRC=\"(.*)\" HEIGHT.*?>/<A HREF=\"$1\" TARGET=\"_blank\">$realm->{'TagLnkIgnore'}<\/A><BR>/gs;
		}

		# Optical - Replace CSS tag
		elsif ($type eq 'optics')
		{
			# Just swap CSS
			${$post} =~ s/PSTNRM/PSTHLT/s;
		}

		# Behead - Replace Taglines and titles
		elsif ($type eq 'behead')
		{
			# Rip out gummy link
			${$post} =~ /<A HREF\=\"(.*\?SEC\=\w*)\" TARGET=\"_blank\">/s;

			# Assign handle
			my $link = $1;


			# Handle image
			${$post} =~ s/<IMG SRC=\"(.*)\" HEIGHT.*?>/<A HREF=\"$1\" TARGET=\"_blank\">$realm->{'TagLnkIgnore'}<\/A><BR>/gs;
			# Removing everything in between
			${$post} =~ s/<\!-HEAD-->.*<\!-POST-->/<A HREF=\"$link\" TARGET=\"_blank\"><IMG SRC=\"$realm->{'SetGummyDefault'}\" BORDER=\"0\" ALIGN=\"LEFT\"><\/A> <FONT CLASS=\"TITLE\">$puppet<\/FONT>/s;
		}
	}
}



#####################
# Handle Posts
#
# Will display all retreived posts to screen, a fairly straightforward 
# operation, will also give a value to NUM once it knows the last post
# received

sub HandlePosts
{
	#####################
	# Data members
	my $self     = shift;					# Self reference
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI Handle
	my $param    = shift;					# Parameter list
	my $choice   = shift;					# Choice handling

	my $statement;						# Query statement

	my $class;						# Command class
	my $cmd;						# Command handling
	my $inline;						# Inline parameters
	my $num;						# Post number
	my $post;						# Post itself
	my $postnum;						# Post number
	my $puppet;						# Puppet name
	my $user;						# User posting

	my %ignore;						# Ignore handle
	my %realm;						# Realm hash


	#####################
	# Program area

	# Link with hash
	$database->GetHashRealm(\%realm);


	# Inline parameters
	$inline = $param->EmbedInline(USER=>$param->{'USER'}, CRYPT=>$param->{'CRYPT'}, ROOM=>$param->{'ROOM'});


	# Retreive posts
	$self->GetPosts($database, \$statement, $param, \%realm);	
	$self->GetIgnore($database, $param, \%ignore);

	# Run though posts and print
	while (($num, $post, $user, $puppet, $cmd, $class) = $statement->fetchrow())
	{ 
		# Additional commands
		if (defined($choice))
		{
			# IC Filter
			if (($user eq $param->{'USER'}) 
			 && ($cmd eq 'play'))
			{
				# Create the 'checkbox'
				$param->{'ICPOSTS'} = 'ON' unless (defined($param->{'ICPOSTS'}));
			}

			# Regular handling
			elsif (($user eq $param->{'USER'})
			 && ($cmd eq 'general'))
			{
				# Remove the 'checkbox'
				$param->Cleanup('ICPOSTS') if (defined($param->{'ICPOSTS'}));
			}
		}


		# Skip
		next if ((defined($param->{'ICPOSTS'})) && ($class eq 'general'));


		# Bio links
		$self->HandleBio($cgi, $inline, \$post);

		# Ignore
		$self->HandleIgnore(\%realm, \$post, $ignore{$puppet}, $puppet);


		# Post Display
		# Display seperator/post
		print $database->FetchRealmSeperator($param->{'ROOM'});

		# Display post
		print "$post\n";


		# Assign Post Number
		$postnum = $num;

		# Assignment of root post
		$param->{'ROOT'} = (defined($param->{'ROOT'})) ? $param->{'ROOT'} : $num;


	}

	# Finish query
	$statement->finish();
	
	# Change value of NUM or define
	if (defined($postnum))
	{ $param->{'NUM'} = ($postnum - 1); }
}


#####################
# Handle Stream
#
# Will display all retreived posts to screen, a fairly straightforward 
# operation.  This unlike HandlePosts will not stop until the stream
# is ended

sub HandleStream
{
	#####################
	# Data members
	my $self     = shift;					# Self reference
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI Handle
	my $param    = shift;					# Parameter list

	my $statement;						# Query statement

	my $class;						# Command class
	my $cmd;						# Command handling
	my $inline;						# Inline parameters
	my $num;						# Post number
	my $post;						# Post itself
	my $puppet;						# Puppet handle
	my $user;						# User posting

	my $postnum;						# Post number
	my $postold = 0;					# Old post number

	my $size    = 0;					# Size of posts
	my $active  = 0;					# Activity counter
	my $elapse  = 0;					# Time elapsed		

	my %ignore;						# Ignore handler
	my %realm;						# Chat hash


	#####################
	# Program area

	# Connect to hash
	$database->GetHashRealm(\%realm);

	# Inline parameters
	$inline = $param->EmbedInline(USER=>$param->{'USER'}, CRYPT=>$param->{'CRYPT'}, ROOM=>$param->{'ROOM'});


	# Commit to remove any shared locks
	$database->Commit();

	# De-activate transactions
	$database->{'HANDLE'}->{'AutoCommit'} = 0;


	# As long as conditions are not met
	while (($size  < $realm{'SetStreamSize'}) 
	   && ($active < $realm{'SetActive'})
	   && ($elapse < $realm{'SetElapse'}))
	{
		# Retreive posts
		$self->GetPosts($database, \$statement, $param, \%realm);	
		$self->GetIgnore($database, $param, \%ignore);

		# Run though posts and print
		while (($num, $post, $user, $puppet, $cmd, $class) = $statement->fetchrow())
		{ 
			# Additional commands
			# IC Filter
			if (($user eq $param->{'USER'}) 
			 && ($cmd eq 'play'))
			{
				# Create the 'checkbox'
				$param->{'ICPOSTS'} = 'ON' unless (defined($param->{'ICPOSTS'}));
			}

			# Regular handling
			elsif (($user eq $param->{'USER'})
			 && ($cmd eq 'general'))
			{
				# Remove the 'checkbox'
				$param->Cleanup('ICPOSTS') if (defined($param->{'ICPOSTS'}));
			}


			# Skip
			next if ((defined($param->{'ICPOSTS'})) && ($class eq 'general'));


			# Bio links
			$self->HandleBio($cgi, $inline, \$post);

			# Ignore
			$self->HandleIgnore(\%realm, \$post, $ignore{$puppet}, $puppet);


			# Display
			# Display seperator
			print $database->FetchRealmSeperator($param->{'ROOM'});

			# Display post
			print "$post\n";
	
			# Assignment of post number
			$postnum = $num;
		
			# Assignment of root post
			$param->{'ROOT'} = (defined($param->{'ROOT'})) ? $param->{'ROOT'} : $num;


			# Possibilities of continuation
			# Determin cumulative size
			if (($user eq $param->{'USER'}) 
			 && ($cmd eq 'stream'))
			{
				# Trick and terminate
				$size  = $realm{'SetStreamSize'};

				# Increment to skip post
				$postnum++;

				# Clear loop
				last;
			}

			# Default behaviour
			else
			{
				# Simply add length
				$size += length($post);
			}

		}

		# Finish query
		$statement->finish();
	
		# Change value of NUM or define
		if (defined($postnum))
		{ 
			# Increment 
			$active = ($postold != $postnum ) ? 0 : ($active + 1); 

			# Assign PostOld and Parameter
			$postold 	= $postnum;
			$param->{'NUM'} = $postnum;
		}

		# Print comment
		print $cgi->comment("$postnum"), "\n";

		# Final size calculations
		$size += (length($postnum) + 8);
		
		# Increment Elapsed time
		$elapse++;


		# Sleep before next pass
		sleep(5);
	}

	# To retrieve last made post
	if (defined($param->{'NUM'})) { $param->{'NUM'}--; }


	# Activation button
	print $cgi->start_form();
	print $param->EmbedNormal($param->Flat(), MAIN=>'True'), "\n";


	# Display refresh
	$self->RefreshStream($database, $cgi, $param);

	# End of form
	print $cgi->end_form(), "\n";


	# Re-activate transactions
	$database->{'HANDLE'}->{'AutoCommit'} = 0;
}


#####################
# Refresh
#
# Quick subroutine that will create the necessary HTML used at the bottom of
# frames and stream variants.   Allows the hosts to make things look all so
# pretty without changing code.

sub Refresh
{
	#####################
	# Data members
	my $self     = shift;					# Self referencing link
	my $choice   = shift;					# Choice of the matter
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter handle

	my $refresh;						# Refresh panel

	my %realm;						# Realm hash


	#####################
	# Program Area

	# Link with has
	$database->GetHashRealm(\%realm);

	# Pull document
	$refresh = $database->FetchRealmRefresh($param->{'ROOM'});
	

	# Prepare
	# Scrollbacks
	my $wscrlback = $cgi->checkbox(-name=>'SCROLLBACK', -label=>$realm{'TxtScrollback'});
	my $wscrladvd = $cgi->textfield('SCROLLBACK', '', 3, 3) . " " . $realm{'TxtScrollback'};

	# Post retreival
	my $wscrlctrl = $cgi->checkbox(-name=>'GETPOSTS', -label=>$realm{'TxtRemote'});

	# Button
	my $wwrefresh =  $cgi->submit(-name=>$realm{'SetPost'}, -value=>$realm{'SetPost'}, -accesskey=>'r');


	# Search and replace
	$refresh =~ s/WSCRLBACK/$wscrlback/gsi;
	$refresh =~ s/WSCRLADVD/$wscrladvd/gsi;
	$refresh =~ s/WSCRLCTRL/$wscrlctrl/gsi;
	$refresh =~ s/WWREFRESH/$wwrefresh/gsi;


	# Display
	print "$refresh\n";
}


# Framed version
sub RefreshFramed
{ Refresh(shift, '0', @_); }

# Stream version
sub RefreshStream
{ Refresh(shift, '1', @_); }

1;								# Return true
